package piglets.beta;

import piglets.*;
import universum.bi.*;

/**
 * Created by IntelliJ IDEA.
 * User: Tanya
 * Date: 02.01.2007
 * Time: 21:05:41
 */
public class Piglet extends SuperPiglet {
    public static final float M0 = 5;
    public static final float S0 = 3;

    public static final float R_crit = 0.2f;
    public static final float R_wait = 7;

    // STATES
    public enum State {
        UNKNOWN,
        EXPLORE,
        GOBBLE,
        SURVIVE,
    }
    private State state = State.UNKNOWN;

    public State getState() {
        return state;
    }

    // explore
    public void setStateExplore() {
        this.state      = State.EXPLORE;
    }

    // gobble
    public void setStateGobble(DrawWell drawWell) {
        this.state      = State.GOBBLE;
        this.drawWell   = drawWell;
    }

    private DrawWell drawWell;

    // HERDER
    private static final Herder herder = new Herder();

    protected SuperHerder getHerder() {
        return herder;
    }

    public void reinit(UserGameInfo info) {
        herder.init(info);
    }

    public Event makeTurn(BeingInterface bi) {
        herder.updateTurn(bi);

        Location pos = bi.getLocation(this);
        float E = bi.getEnergy(this);
        PointInfo pointInfo = bi.getPointInfo(this);

        // check if we can born
        if (E > M * Constants.K_toborn) {
            return new Event(EventKind.ACTION_BORN, getParams());
        }

        Location next;
        switch (state) {
            case EXPLORE:
                next = findExploreStep(pos, bi);
                if (next != null) {
                    return new Event(EventKind.ACTION_MOVE_TO, next);
                }

                // check if we can eat anything to born
                if (pointInfo.getCount(this) + pointInfo.getGrowthRate(this) * R_wait > M * Constants.K_toborn - E) {
                    return new Event(EventKind.ACTION_EAT, M - E);
                }

                for (PointInfo pi : bi.getNeighbourInfo(this)) {
                    if (pi.getCount(this) + pi.getGrowthRate(this) * R_wait > M * Constants.K_toborn - E) {
                        return new Event(EventKind.ACTION_MOVE_TO, pi.getLocation());
                    }
                }

                state = State.UNKNOWN;
                break;

            case GOBBLE:
                next = findStep(pos, drawWell.pos);
                if (next != null) {
                    return new Event(EventKind.ACTION_MOVE_TO, next);
                }

                if (pointInfo.getCount(this) + pointInfo.getGrowthRate(this) * R_wait > M * Constants.K_toborn - E) {
                    return new Event(EventKind.ACTION_EAT, M - E);
                }

                // otherwise, just survive
                if (E < M * R_crit) {
                    return new Event(EventKind.ACTION_EAT, 2 * M * Constants.K_masscost);
                }
                break;

            case UNKNOWN:
                // just do nothing
                break;
        }

        return null;
    }

    private static final int [] EXPLORE_DELTA = new int [] { 0, 1, -1, 2, -2, 3, -3 };
    private Location findExploreStep(Location pos, BeingInterface bi) {
        final int VARIATION_LENGTH = 3;

        int max = 0;
        Location best = null;

        for (int ED_i : EXPLORE_DELTA) {
            for (int ED_j : EXPLORE_DELTA) {
                // let's calc cells amount that will be explored
                int unexplored = 0;

                synchronized (herder) {
                    for (int di = 0; di < VARIATION_LENGTH; ++di) {
                        for (int dj = 0; dj < VARIATION_LENGTH; ++dj) {
                            int x = normalizeX(pos.getX() + ED_i + EXPLORE_DELTA[di]);
                            int y = normalizeY(pos.getY() + ED_j + EXPLORE_DELTA[dj]);

                            if (!herder.glade[x][y].explored && herder.glade[x][y].shouldBeExplored != herder.getTurn()) {
                                ++unexplored;
                            }
                        }
                    }
                }

                if (max < unexplored) {
                    max = unexplored;
                    best = new Location(
                            normalizeX(pos.getX() + ED_i),
                            normalizeY(pos.getY() + ED_j));
                }
            }
        }

        if (best != null) {
            synchronized (herder) {
                // mark that cells that it will be explored
                    for (int di = 0; di < VARIATION_LENGTH; ++di) {
                        for (int dj = 0; dj < VARIATION_LENGTH; ++dj) {
                            int x = normalizeX(best.getX() + EXPLORE_DELTA[di]);
                            int y = normalizeY(best.getY() + EXPLORE_DELTA[dj]);

                            herder.glade[x][y].shouldBeExplored = herder.getTurn();
                        }
                    }
            }
        }

        return best;
    }

    public BeingParams getParams() {
        return new BeingParams(M0, S0);
    }
}
