package universum;

import universum.engine.*;
import universum.ui.*;
import universum.bi.Constants;
import universum.bi.GameKind;

import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import java.io.*;
import java.util.*;

/**
 *
 * @author nike
 */
public class Main {
    public  JFrame frame;
    private String viewFile;
    private boolean headless = false;
    private String  headlessArgs;
    private boolean altui = false;
    private boolean secure = false;
    private GameKind gameKind = GameKind.JUNGLE;

    public Main() {
    }
    
    private JComponent createCtr() {
        if (viewFile != null) {
            try {
                return new LocalViewer(viewFile);
            } catch (IOException ioe) {
                ioe.printStackTrace();
                return null;
            }
        } else {
            return new Controller(frame, gameKind);
        }
    }
    
    private void createMainFrame() {
        if (altui) {
             universum.ui.sam.Application app = 
                 new universum.ui.sam.Application();
             app.run();
             this.frame = app.getFrame();
             return;
        }

        frame = new JFrame("Universum");
        final int WIDTH = 900, HEIGHT = 600;
        frame.setSize(WIDTH, HEIGHT);
        Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
        frame.setLocation(d.width/2 - WIDTH/2, d.height/2 - HEIGHT/2);

        final JComponent ctr = createCtr();

        frame.addWindowListener( new WindowAdapter() {
                public void windowOpening( WindowEvent e ) {
                    ctr.repaint();
                }
                
                public void windowClosing( WindowEvent e ) {
                    frame.dispose();
                    System.exit(0);
                }
            });
        
        
        Container c = frame.getContentPane();
        c.setLayout(new BorderLayout());
        c.add(ctr, BorderLayout.CENTER);
        frame.validate();
        frame.setVisible(true);
        frame.repaint();
    }
    
    void proceed(String[] args) {
        int i = 0;
        
        boolean gotcha = true;
        while  (i < args.length) {
            if ("-record".equals(args[i]) && args.length > i + 1) {
                i++;
                Constants.setRecordFile(args[i++]);
                continue;
            }

            if ("-view".equals(args[i]) && args.length > i + 1) {
                i++;
                viewFile = args[i++];
                continue;
            }
            if ("-headless".equals(args[i]) && args.length > i + 1) {
                i++;
                headless = true;
                headlessArgs =  args[i++];
                continue;
            }

            if ("-kind".equals(args[i])) {
                i++;
                try {
                    gameKind = GameKind.valueOf(args[i++]);
                } catch (IllegalArgumentException iae) {
                    System.out.println("Unknown game: " +args[i-1]+
                                       "; known game kinds are:");
                    for (GameKind gk : GameKind.values()) {
                        System.out.println(" "+gk);
                    }
                    System.exit(1);
                }
                continue;
            }

            if ("-altui".equals(args[i])) {
                i++;
                altui = true;
                continue;
            }

            if ("-debug".equals(args[i])) {
                i++;
                Constants.setDebug(true);
                continue;
            }

            if ("-secure".equals(args[i])) {
                i++;
                JungleSecurity.setCheckSecurity(true);
                continue;
            }
            
            if ("-makebeing".equals(args[i])) {
                i++;
                makeBeing(args[i++]);
                System.exit(0);
                continue;
            }
            

            gotcha = false;
            i++;
        }

        if (!gotcha) {
            System.out.print("Known command line flags are:\n"+
                             " -headless class1@file1.jar,class2@file2.jar - runs contest in batch mode with no visualization\n"+
                             " -record file - records game to file\n"+
                             " -view file - plays back recorded game\n"+
                             " -altui - use alternative user interface\n"+
                             " -debug - debugging info\n"+
                             " -kind - set game kind\n"+
                             " -makebeing my.properties - make being by description" 
                               );
            return;                               
        }
        
        assert gameKind != null;
        
        if (headless) {
            HeadlessController ctr = new HeadlessController(gameKind, -1);
            StringTokenizer st = new StringTokenizer(headlessArgs, ",");
            java.util.List<String[]> hargs = new ArrayList<String[]>();
            while (st.hasMoreTokens()) {                
                String n = st.nextToken();                
                int at = n.indexOf('@');
                if (at == -1) {
                    System.err.println("-headless arg must be: class1.name@file1.jar,class2.name@file2.jar");
                    System.exit(1);
                }
                hargs.add(new String[] { n.substring(0, at), 
                                         n.substring(at+1, n.length()) });
            }
            ctr.startGame(hargs.toArray(new String[hargs.size()][]));
        } else {
            SwingUtilities.invokeLater(new Runnable() {
                    public void run() {
                        createMainFrame();
                    }
                });
        }
    }
    
    public static void main(final String[] args) {
        new Main().proceed(args);
    }   

    private static boolean needRebuild(String jar, String bhome, String files) {
        File fjar = new File(jar);
        if (!fjar.exists()) {
            return true;
        }
        long jmfile = fjar.lastModified();
        StringTokenizer st = new StringTokenizer(files, ",");        
        while (st.hasMoreTokens()) {
            File f = new File(bhome+"/"+st.nextToken());
            if (f.lastModified() > jmfile) {
                return true;
            }
        }
        return false;
    }
   
    public static boolean makeBeing(String file) {
        boolean done = false;
        try {
            Properties props = new Properties();
            File ffile = new File(file);
            if (!ffile.exists()) {
                ffile = new File("../src/universum/beings", ffile.getName());
            }
            if (!ffile.exists()) {
                System.out.println("Cannot find being descriptor: "+file);
                return false;
            }
            InputStream is = new FileInputStream(ffile);
            props.load(is);
            String bhome = ffile.getParentFile().getAbsolutePath();
            String jar = beingPath(props.getProperty("jar"));

            if (!needRebuild(jar, bhome, props.getProperty("files"))) {
                System.out.println(jar+" already up to date");
                return true;
            }

            Tool t = Tool.getTool();
            File being = new File(new File(jar).getParentFile(), "being");
            t.rmdir(being);
            being.mkdir();
            
            java.util.List<String> args = new java.util.ArrayList<String>();
            args.add("-target");  args.add("1.5"); 
            args.add("-classpath");
            String cp1 = 
                new File(being, "../dist").getAbsolutePath();
            String cp2 = 
                new File(".", "universum.jar").getAbsolutePath();
            // very special case, to handle building of SimpleBeing in Netbeans
            String cp3 = 
                new File(new File(file).getParentFile(), "../../../build/classes").getAbsolutePath();
                
            args.add(cp1+File.pathSeparator+cp2+File.pathSeparator+cp3);
            args.add("-d"); args.add(being.getAbsolutePath());
                       
            StringTokenizer st = new StringTokenizer(props.getProperty("files"), ",");
            while (st.hasMoreTokens()) {
                args.add(bhome+"/"+st.nextToken());                
            }
            
            int rv = t.compile(args.toArray(new String[args.size()]));            
            if (rv != 0) {
                System.out.println("Compilation error");
                return false;
            }            

            // now make the jar
            File man = new File(being, "BEING.MF"); 
            FileWriter manw = new FileWriter(man);
            String main = "Main-Class: "+props.getProperty("main-class") + "\r\n"; 
            manw.write(main, 0, main.length());
            manw.close();

            args.clear();
            args.add("cmf"); args.add(man.getAbsolutePath());
            args.add(jar);
            args.add("-C");  args.add(being.getAbsolutePath());
            args.add(".");
            rv = t.jar(args.toArray(new String[args.size()]));
            man.delete();           

            if (rv == 0) {
                System.out.println("Sucessfully made: "+jar);
            } else {
                System.out.println("Packing error");
            }
            t.rmdir(being);
            done = true;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return done;
    }

    private static String beings;
    public static String beingPath(String name) {
        if (beings == null) {
            File b = new File("../beings");
            b.mkdir();
            try {
                beings = b.getCanonicalPath() + File.separator;
            } catch (IOException ioe) {}
        }

	if (new File(name).exists()) {
            return name;
        }

        return beings+name;
    }
}
