package universum.engine;

import java.util.*;
import universum.bi.Location;
import universum.bi.Constants;

import universum.util.Util;
import universum.util.Walker;

/**
 *
 * @author nike
 */
public class DefaultResourceControl implements ResourceControl {
    protected Map<Location,float[]> resources;
    protected Universe universe;
    private List<Location> zombies;
    protected int reg, golden;
    
    /** Creates a new instance of DefaultResourceControl */
    public DefaultResourceControl(int reg, int golden) {
        resources = new HashMap<Location,float[]>();
        zombies = new LinkedList<Location>();
	this.reg = reg;
	this.golden = golden;
    }
    
    public void init(Universe is42) {
        universe = is42;
        
	// add regular feeding grounds
        for (int i = 0; i<reg; i++) {
            addNextResource(universe.getTopo().getRandomLocation(), false);
        }
	
	// and few golden spots
	for (int i = 0; i<golden; i++) {
	    addNextResource(universe.getTopo().getRandomLocation(), true);
	}
    }
        
    protected void addNextResource(Location loc, boolean golden) {
        addSource(loc, 
                  Util.frnd(golden ? 
                            Constants.MAX_GOLDEN : Constants.MAX_REGULAR), 
                  Util.frnd(golden ? 
                            Constants.GROW_GOLDEN : Constants.GROW_REGULAR), 
                  /* same as count */
                  golden ? Constants.MAX_GOLDEN : -1f);
    }

    private float[] get(Location loc) {
        synchronized (resources) {
           return resources.get(loc); 
        }
    } 
    
    public float getCount(Location loc) {
        float[] res = get(loc);
        return  res == null ? 0f : res[0];
    }
    
    public float getGrowthRate(Location loc) {
        float[] res = get(loc);        
        return  res == null ? 0f : res[1];
    }
    
    public float getMaxCount(Location loc) {
        float[] res = get(loc);        
        return  res == null ? 0f : res[2];
    }
    
    public float consume(Location loc, float count) {
        float[] res = get(loc);
        float c = 0f;
        
        if (res != null) {
            synchronized (res) {
                c = res[0];
                if (c >= count) {
                    res[0] -= count;
                    c = count;
                } else {
                    c = res[0];
                    res[0] = 0f;
                }
            }
        }        
        return c;
    }
    
    public void forAllResourceLocations(Walker<Location> walker) {      
        synchronized (resources) {
            Util.walkList(resources.keySet(), walker);
        }
    }
    
    public void addSource(Location loc, float count, float rate, float max) {
       synchronized (resources) {
            float res[] = resources.get(loc);
	    // normalization
	    if (max < 0f) max = count;
	    if (count > max) count = max;

            if (res != null) {
                res[0] += count;
                res[1] += rate;
            } else {
               res = new float[] {count, rate, max};
               resources.put(loc, res);
            }
            if (res[0] > res[2]) res[0] = res[2];            
        }
    }
    
    public void removeSource(Location loc) {
        synchronized (resources) {
            //resources.put(loc, null);
            resources.remove(loc);
        }
    }
    
    public void makeTurn() {
        zombies.clear();
        synchronized (resources) {
            Set<Location> places = resources.keySet();
            for (Location l : places) {
                float res[] = resources.get(l);
                res[0] += res[1];
                if (res[0] == 0f && res[1] == 0f) {
                    zombies.add(l);
                }
		if (res[0] > res[2]) {
		    res[0] = res[2];
		}
            }
            for (Location z : zombies) {
                resources.remove(z);
            }
        }
    }
}
